@@ -23,7 +23,7 @@ import bossa.syntax.LocatedString;
    An abstract package source, where source or interface files 
    and compiled code if applicable can be found.
    
-   @version $Date: 2004/08/10 17:22:19 $
+   @version $Date: 2005/06/18 13:26:55 $
    @author Daniel Bonniot
  */
 
@@ -67,7 +67,11 @@ class Content
          lastModification > lastCompilation))
       readers = source.getDefinitions();
     else
-      readers = compiled.getDefinitions();
+      {
+        pkg.compiledModule =
+          new bossa.syntax.Module(pkg, pkg.getName(), pkg.packageScope);
+        readers = compiled.getDefinitions();
+      }
 
     LinkedList imports = new LinkedList();
 
@@ -108,7 +112,24 @@ class Content
   private void read(Content.Unit unit, List definitions)
   {
     bossa.util.Location.setCurrentFile(unit.file);
-    pkg.compilation.parser.read(unit.reader, definitions);
+
+    bossa.syntax.Module module;
+
+    if (pkg.compiledModule != null)
+      module = pkg.compiledModule;
+    else
+      {
+        nice.tools.visibility.Scope scope =
+          new nice.tools.visibility.Scope(unit.getName(), pkg.packageScope);
+
+        module = new bossa.syntax.Module(pkg, unit.name, scope);
+      }
+
+    Definition.currentModule = module;
+
+    pkg.compilation.parser.read(unit.reader, module, definitions);
+
+    Definition.currentModule = null;
   }
 
   /**
@@ -252,6 +273,15 @@ class Content
       this.reader = reader;
       this.name = name;
     }
+
+    public String getName()
+    {
+      if (file != null)
+        return file.toString();
+      else
+        return name;
+    }
+
     BufferedReader reader;
     String name;
     File file;
@@ -264,43 +294,34 @@ class Content
       this.stream = stream;
       this.name = name;
     }
+
+    public boolean equals(Object o)
+    {
+      return o instanceof Stream && name.equals(((Stream) o).name);
+    }
+
+    public int hashCode() { return name.hashCode(); }
+
     InputStream stream;
     String name;
   }
 
   /**
-     Returns the compiled classes of this source. 
-     
-     This method is valid for sources that were not compiled
-     because they were up-to-date.
-     It returns the classes generated by the compilation of the
-     package denoted by this source.
+     Returns the compiled classes of this source.
   */
   Stream[] getClasses(boolean linkPerformed)
   {
-    Stream[] res;
+    /* We first add the generated classes, then all those compiled classes
+       that were not regenerated. The Set ensures that the previous version of
+       regenerated classes is not included.
+    */
+    Set res = new HashSet();
+
+    DirectoryCompiledContent.addClasses(res, getOutputDirectory());
 
     if (!sourceRead)
-      {
-	res = compiled.getClasses(! linkPerformed);
-	if (linkPerformed)
-	  {
-	    File dispatchFile = new File(getOutputDirectory(), 
-					 "dispatch.class");
-	    try{
-	      res[res.length - 1] = new Stream
-		(new FileInputStream(dispatchFile), "dispatch.class");
-	    }
-	    catch(FileNotFoundException e) {
-	      Internal.error(pkg + 
-			     ": dispatch class could not be added to archive" +
-			     "\nI expected it to be in " + dispatchFile);
-	    }
-	  }
-      }
-    else
-      res = DirectoryCompiledContent.getClasses(getOutputDirectory(), true);
+      compiled.addClasses(res);
 
-    return res;
+    return (Stream[]) res.toArray(new Stream[res.size()]);
   }
 }
