@@ -20,7 +20,7 @@ import gnu.bytecode.ClassType;
 /**
    A package located in a directory.
 
-   @version $Date: 2004/07/02 19:31:03 $
+   @version $Date: 2005/08/27 14:04:40 $
    @author Daniel Bonniot
  */
 
@@ -119,6 +119,17 @@ class DirectorySourceContent extends SourceContent
   private BufferedReader read(File f)
   {
     try{
+      String encoding = pkg.compilation.sourceEncoding;
+      if(encoding != null) try{
+        FileInputStream fis = new FileInputStream(f);
+        // No need to wrap the FileInputStream with BufferedInputStream here:
+        // InputStreamReader already contains an input buffer, both in Sun and GNU implementations.
+        InputStreamReader isr = new InputStreamReader(fis, encoding);
+        return new BufferedReader(isr);
+      }catch(UnsupportedEncodingException badEncoding){
+        User.warning("Encoding '" + encoding + "' was rejected while reading " +
+                     nice.tools.util.System.prettyPrint(f));
+      }
       return new BufferedReader(new FileReader(f));
     }
     catch(FileNotFoundException e){
