@@ -29,10 +29,10 @@ import gnu.expr.ClassExp;
 /**
    A Nice package.
    
-   @version $Date: 2005/02/20 21:20:02 $
+   @version $Date: 2005/03/24 14:47:07 $
    @author Daniel Bonniot (bonniot@users.sourceforge.net)
  */
-public class Package implements mlsub.compilation.Module, Located, bossa.syntax.Module
+public class Package implements mlsub.compilation.Module, Located
 {
   /****************************************************************
    * Loading
@@ -73,6 +73,8 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
 
     compilation.packages.put(name.toString(), this);
 
+    packageScope = new nice.tools.visibility.Scope(getName(), null);
+
     source = compilation.locator.find(this);
     if (source == null)
       User.error(name, "Could not find package " + name);
@@ -98,13 +100,20 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
 
     if (!name.toString().equals("nice.lang") && !Debug.ignorePrelude)
       imports.add(new LocatedString("nice.lang",
-				    bossa.util.Location.nowhere()));
+                                    bossa.util.Location.nowhere()));
 
     setOpens(opens);
 
     List p = this.getImports();
     for (Iterator i = p.iterator(); i.hasNext();)
-      source.someImportModified(((Package) i.next()).lastModification());
+      {
+        Package imported = (Package) i.next();
+        source.someImportModified(imported.lastModification());
+        if (imported != this)
+          packageScope.addImplicitOpen(imported.packageScope);
+      }
+
+    packageScope.addImplicitOpen(compilation.javaScope);
   }
 
   /**
@@ -112,10 +121,8 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
    **/
   private void read(boolean shouldReload)
   {
-    Module oldModule = Definition.currentModule;
-    Definition.currentModule = this;
-    Node.setModule(this);
-    
+    bossa.syntax.Node.setPackage(this);
+
     compilation.progress(this, "parsing");
 
     definitions = new ArrayList();
@@ -127,10 +134,8 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
     if (compiling())
       // Inform compilation that at least one package is going to generate code
       compilation.recompilationNeeded = true;
-
-    Definition.currentModule = oldModule;
   }
-  
+
   void setOpens(Set opens)
   {
     // when we import a Nice package, we also open it.
@@ -154,8 +159,8 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
     for(Method method = source.getBytecode().getMethods();
 	method != null;
 	method = method.getNext())
-      bossa.syntax.dispatch.readImportedAlternative(source.getBytecode(), method,
-					  location());
+      bossa.syntax.dispatch.readImportedAlternative
+        (source.getBytecode(), method, location(), compiledModule);
   }
   
   /****************************************************************
@@ -194,7 +199,9 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
     for(Iterator i = imports.iterator(); i.hasNext();)
       {
 	LocatedString s = (LocatedString) i.next();
-	importedPackages.add(make(s, compilation, false));
+        Package p = make(s, compilation, false);
+        if (! importedPackages.contains(p))
+          importedPackages.add(p);
       }
   }
 
@@ -231,37 +238,69 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
 
   private boolean scoped = false;
 
+  static nice.tools.util.Chronometer scopeChrono = nice.tools.util.Chronometer.make("Scope");
+  static nice.tools.util.Chronometer resolveChrono = nice.tools.util.Chronometer.make("Resolve");
+  static nice.tools.util.Chronometer readAltChrono = nice.tools.util.Chronometer.make("Read alternatives");
+  static nice.tools.util.Chronometer tresolveChrono = nice.tools.util.Chronometer.make("Typed resolve");
+  static nice.tools.util.Chronometer lresolveChrono = nice.tools.util.Chronometer.make("Local resolve");
+  static nice.tools.util.Chronometer typecheckChrono = nice.tools.util.Chronometer.make("Typecheck");
+
   public void scope()
   {
     if (scoped) return;
     scoped = true;
 
-    ast.buildScope();
+    scopeChrono.start();
+    try {
+      ast.buildScope();
+    } finally {
+      scopeChrono.stop();
+    }
 
     addProgress(PROGRESS_SCOPE);
   }
   
   public void load()
   {
-    ast.resolveScoping();
+    resolveChrono.start();
+    try {
+      ast.resolveScoping();
+    } finally {
+      resolveChrono.stop();
+    }
 
-    // this must be done before freezing
-    if (!compiling())
-      readAlternatives();
+    readAltChrono.start();
+    try {
+      // this must be done before freezing
+      if (!compiling())
+        readAlternatives();
+    } finally {
+      readAltChrono.stop();
+    }
 
     addProgress(PROGRESS_LOAD);
   }
 
   public void typedResolve()
   {
-    ast.typedResolve();
+    tresolveChrono.start();
+    try {
+      ast.typedResolve();
+    } finally {
+      tresolveChrono.stop();
+    }
 
     addProgress(PROGRESS_TYPED_RESOLVE);
   }
 
   public void localResolve()
   {
-    ast.localResolve();
+    tresolveChrono.start();
+    try {
+      ast.localResolve();
+    } finally {
+      tresolveChrono.stop();
+    }
 
     addProgress(PROGRESS_LOCAL_RESOLVE);
   }
@@ -324,8 +363,13 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
     // this pass.
     if (compiling())
       compilation.progress(this, "typechecking");
-    
-    ast.typechecking(compiling());
+
+    typecheckChrono.start();
+    try {
+      ast.typechecking(compiling());
+    } finally {
+      typecheckChrono.stop();
+    }
 
     addProgress(PROGRESS_TYPECHECK);
   }
@@ -533,7 +577,7 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
     // add individual classes
     String[] classes = 
     {
-      "gnu/mapping/Procedure.class", "gnu/mapping/Procedure0.class", "gnu/mapping/Procedure1.class", "gnu/mapping/Procedure2.class", "gnu/mapping/Procedure3.class", "gnu/mapping/ProcedureN.class", "gnu/mapping/Named.class", "gnu/mapping/Printable.class", "gnu/mapping/WrongArguments.class", "gnu/mapping/WrongType.class", "gnu/mapping/WrappedException.class", 
+      "gnu/mapping/Procedure.class", "gnu/mapping/Procedure0.class", "gnu/mapping/Procedure1.class", "gnu/mapping/Procedure2.class", "gnu/mapping/Procedure3.class", "gnu/mapping/ProcedureN.class", "gnu/mapping/Named.class", "gnu/mapping/Printable.class", "gnu/mapping/WrongArguments.class",
       "gnu/expr/ModuleBody.class", "gnu/expr/ModuleMethod.class"
     };
 
@@ -740,9 +784,12 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
     return res;
   }
 
-  public Method lookupDispatchClassMethod(String name, String attribute, String value)
+  public Method lookupDispatchClassMethod
+    (ClassType clas, String name, String attribute, String value)
   {
-    return lookupClassMethod(source.getDispatch(), name, attribute, value);
+    if (clas == null)
+      clas = source.getDispatch();
+    return lookupClassMethod(clas, name, attribute, value);
   }
 
   /**
@@ -827,7 +874,11 @@ public class Package implements mlsub.compilation.Module, Located, bossa.syntax.
   /** The compilation that is in process. */
   Compilation compilation;
   
-  public Compilation compilation() { return compilation; }
+  public Compilation getCompilation() { return compilation; }
+
+  bossa.syntax.Module compiledModule = null;
+
+  nice.tools.visibility.Scope packageScope;
 
   /** 
       @return true if this package was loaded from an interface file,
