@@ -18,7 +18,7 @@ import java.util.*;
    Abstract definition.
    May be a class definition, a method definition, an interface definition...
 
-   @version $Date: 2004/11/26 20:26:02 $
+   @version $Date: 2005/03/06 01:34:26 $
    @author Daniel Bonniot (d.bonniot@mail.dotcom.fr)
  */
 public abstract class Definition extends Node implements bossa.util.Located
@@ -69,7 +69,7 @@ public abstract class Definition extends Node implements bossa.util.Located
 
   public boolean inInterfaceFile()
   {
-    return module.interfaceFile();
+    return module.compiled();
   }
 
   bossa.modules.Compilation compilation()
