@@ -1,6 +1,6 @@
 package mlsub.typing.lowlevel;
 
-import java.util.Vector;
+import java.util.ArrayList;
 
 /**
  * A lowlevel constraint on integers.
@@ -76,7 +76,7 @@ public final class K0 {
     this.garbage = new BitVector();
     this.posTagged = new BitVector();
     this.negTagged = new BitVector();
-    this.interfaces = new Vector();
+    this.interfaces = new ArrayList();
     if (debugK0) {
       System.err.println("created K0 #" + ID);
     }
@@ -387,7 +387,7 @@ public final class K0 {
     C.setSize(n);
     Ct.setSize(n);
     if (domains != null) {
-      domains.setSize(n - m);
+      domains.truncate(n - m);
     }
     garbage.truncate(n);
     posTagged.truncate(n);
@@ -468,7 +468,7 @@ public final class K0 {
   }
 
   // a vector of all the Interfaces in this constraint
-  private Vector interfaces;
+  private ArrayList interfaces;
   /**
    * Returns the number of interfaces. Interfaces are garanteed to be
    * numbered from 0 to nInterfaces()-1
@@ -479,7 +479,7 @@ public final class K0 {
 
   // assume 0 <= iid < nInterfaces()
   Interface getInterface(int iid) {
-    return (Interface)interfaces.elementAt(iid);
+    return (Interface)interfaces.get(iid);
   }
 
   /***********************************************************************
@@ -496,7 +496,7 @@ public final class K0 {
     S.assume(S.a&& !hasBeenInitialized);
     int iid = nInterfaces();
     Interface iface = new Interface(this, iid);
-    interfaces.addElement(iface);
+    interfaces.add(iface);
     if (debugK0) {
       System.err.println("newInterface in #" + ID + " -> " + iid);
     }
@@ -549,9 +549,9 @@ public final class K0 {
     
     // put in R and Rt, the constraint saturated under
     // x < y and y < z => x < z
-    R = (BitMatrix)C.clone();
+    R = new BitMatrix(C);
     R.closure();
-    Rt = (BitMatrix)Ct.clone();
+    Rt = new BitMatrix(Ct);
     Rt.closure();
     m0 = m = n;
     
@@ -593,6 +593,7 @@ public final class K0 {
    * Assume x is a valid index and iid is a valid interface id
    **/
   public void indexImplements(int x, int iid) throws Unsatisfiable {
+    usingInterfaces = true;
     S.assume(S.a&& hasBeenInitialized);
     if (LowlevelUnsatisfiable.refinedReports) {
       try {
@@ -678,7 +679,10 @@ public final class K0 {
     // makes it unnecessary to mark-backtrack in some cases
     if (isRigid(x1) && isRigid(x2)) {
       if (!R.get(x1, x2)) {
-        throw new LowlevelRigidClash(indexToString(x1), indexToString(x2));
+        if (debugK0)
+          throw new LowlevelRigidClash(indexToString(x1), indexToString(x2));
+        else
+          throw LowlevelUnsatisfiable.instance;
       } else {
         return;
       }
@@ -825,13 +829,13 @@ public final class K0 {
     S.assume(S.a&& x >= -1);
     if (x == -1) {
       if (!unit) {
-        throw new LowlevelUnsatisfiable();
+        throw LowlevelUnsatisfiable.instance;
       }
     } else if (x < m) {
       // the domain must contain x itself (we assume here that the relation
       // is condensed on the rigid variables)
       if (!set.get(x)) {
-        throw new LowlevelUnsatisfiable();
+        throw LowlevelUnsatisfiable.instance;
       }
     } else {
       if(debugK0){
@@ -1078,7 +1082,7 @@ public final class K0 {
   private void condense()
     throws Unsatisfiable
   {
-    BitMatrix T=(BitMatrix)C.clone();
+    BitMatrix T = new BitMatrix(C);
     T.closure();
     condense(T);
   }
@@ -1105,10 +1109,10 @@ public final class K0 {
     if (m > 0) {
       if (T.includedIn(m, R) != null) {
         // T is NOT included in R on [0, m[ x [0, m[
-        throw new LowlevelUnsatisfiable();// will be refined if necessary
+        throw LowlevelUnsatisfiable.instance; // will be refined if necessary
       }
     }
-    BitMatrix Tt = (BitMatrix)Ct.clone();
+    BitMatrix Tt = new BitMatrix(Ct);
     Tt.closure();
 
     if(debugK0){
@@ -1154,13 +1158,18 @@ public final class K0 {
     collect();
   }
 
+  private boolean usingInterfaces = false;
+
   private void prepareConstraint() throws Unsatisfiable {
     collapseMinimal();
-    BitMatrix leq = (BitMatrix)C.clone();
-    leq.closure();
-    computeArrows(leq);
-    saturateAbs(leq);
-    //condense(leq);
+    if (usingInterfaces) {
+      BitMatrix leq = new BitMatrix(C);
+      leq.closure();
+      computeArrows(leq);
+      saturateAbs(leq);
+      //condense(leq);
+      usingInterfaces = false;
+    }
   }
 
 
@@ -1271,7 +1280,7 @@ public final class K0 {
     BitVector[] rigidImplementors = new BitVector[nInterfaces()];
     for (int iid = 0; iid < nInterfaces(); iid++) {
       BitVector I_impls = getInterface(iid).implementors;
-      rigidImplementors[iid] = (BitVector)I_impls.clone();
+      rigidImplementors[iid] = new BitVector(I_impls);
       for (int x = I_impls.getLowestSetBit();
            x != BitVector.UNDEFINED_INDEX;
            x = I_impls.getNextBit(x)) {
@@ -1300,9 +1309,9 @@ public final class K0 {
    **/
   public void rigidify() {
     S.assume(S.a&& hasBeenInitialized);
-    R = (BitMatrix)C.clone();
+    R = new BitMatrix(C);
     R.closure();
-    Rt = (BitMatrix)Ct.clone();
+    Rt = new BitMatrix(Ct);
     Rt.closure();
     m = n;
     BitVector[] rigidImplementors  = closeImplements(R, Rt);
@@ -1355,10 +1364,10 @@ public final class K0 {
       // We only need to save C if there are soft variables, since
       // others can't be modified anyway.
       if (K0.this.m != K0.this.n)
-        this.savedC = (BitMatrix) K0.this.C.clone();
+        this.savedC = new BitMatrix(K0.this.C);
 
-      this.savedGarbage = (BitVector)K0.this.garbage.clone();
-      this.savedDomains = (DomainVector)K0.this.domains.clone();
+      this.savedGarbage = new BitVector(K0.this.garbage);
+      this.savedDomains = new DomainVector(K0.this.domains);
       /*
       this.savedImplementors = new BitVector[K0.this.nInterfaces()];
       for (int iid = 0; iid < savedImplementors.length; iid++) {
@@ -1653,9 +1662,9 @@ public final class K0 {
     Simplifier(BitVector simplified) {
       this.simplified = simplified;
       this.initN = simplified.getLowestSetBit();
-      R = (BitMatrix)C.clone();
+      R = new BitMatrix(C);
       R.closure();
-      Rt = (BitMatrix)Ct.clone();
+      Rt = new BitMatrix(Ct);
       Rt.closure();
 
       implementors = closeImplements(R, Rt);
@@ -1791,7 +1800,7 @@ public final class K0 {
       while (excludedA >= initN) {
         if (!garbage.get(excludedA) && simplified.get(excludedA)) {
           int x;
-          DomainVector savedDomains = (DomainVector)Sdomains.clone();
+          DomainVector savedDomains = new DomainVector(Sdomains);
           try {
             if (S.debugSimpl) {
               System.err.println("Try excluding " + excludedA);
