@@ -1,12 +1,12 @@
 package mlsub.typing.lowlevel;
 
 /**
- * @version $Revision: 1.1 $, $Date: 2000/06/14 13:32:53 $
+ * @version $Revision: 1.3 $, $Date: 2005/05/25 23:22:57 $
  * @author Alexandre Frey
  **/
 public class LowlevelUnsatisfiable extends Unsatisfiable 
 {
-  LowlevelUnsatisfiable(String msg)
+  public LowlevelUnsatisfiable(String msg)
   {
     super(msg);
   }
@@ -16,6 +16,8 @@ public class LowlevelUnsatisfiable extends Unsatisfiable
     this("[NO MESSAGE]");
   }
   
+  public static LowlevelUnsatisfiable instance = new LowlevelUnsatisfiable();
+
   static boolean refinedReports = true;
   public static void setRefinedReports(boolean flag) {
     refinedReports = flag;
