@@ -22,7 +22,7 @@ import java.util.*;
  * Class represents a nice source file.
  * 
  * @author	Alex Greif <a href="mailto:alex.greif@web.de">alex.greif@web.de</a>
- * @version	$Id: NiceSourceFile.java,v 1.19 2004/08/05 12:33:21 bonniot Exp $
+ * @version	$Id: NiceSourceFile.java,v 1.21 2005/06/09 07:54:28 bonniot Exp $
  */
 public class NiceSourceFile {
 	/**
@@ -80,8 +80,8 @@ public class NiceSourceFile {
 	 * TODO
 	 * 
 	 */
-	private Set _imports = new HashSet();
-	
+	private List _imports = new LinkedList();
+
 	/**
 	 * TODO
 	 * 
@@ -345,7 +345,7 @@ public class NiceSourceFile {
 			return;
 			
 		writer.write("// main method");writer.newLine();
-		writer.write("void main(java.lang.String[] args) {\n");
+		writer.write("void main(java.lang.String[] _args) {\n");
 		writer.write(_mainMethodContent.toString());
 		writer.write("}\n");
 	}
