@@ -22,7 +22,7 @@ import java.util.*;
  * A testsuite file with its testcases.
  * 
  * @author	Alex Greif <a href="mailto:alex.greif@web.de">alex.greif@web.de</a>
- * @version	$Id: TestSuite.java,v 1.17 2005/03/21 21:25:57 bonniot Exp $
+ * @version	$Id: TestSuite.java,v 1.18 2007/11/25 20:03:14 bonniot Exp $
  */
 public class TestSuite {
 
@@ -30,25 +30,25 @@ public class TestSuite {
 	 * TODO
 	 * 
 	 */
-	private final String TESTCASE_TYPE_PASS = "pass";
+	private static final String TESTCASE_TYPE_PASS = "pass";
 
 	/**
 	 * TODO
 	 * 
 	 */
-	private final String TESTCASE_TYPE_FAIL = "fail";
+	private static final String TESTCASE_TYPE_FAIL = "fail";
 
 	/**
 	 * TODO
 	 * 
 	 */
-	private final String KEYWORD_GLOBAL = "global";
+	private static final String KEYWORD_GLOBAL = "global";
 
 	/**
 	 * TODO
 	 * 
 	 */
-	private final String KEYWORD_COMMENT = "comment";
+	private static final String KEYWORD_COMMENT = "comment";
 
 
 	/**
