@@ -22,7 +22,7 @@ import gnu.bytecode.*;
    Do no throw a java.lang.AssertionError, since that class is only 
    available in JDK 1.4 and later. Instead throw a nice.lang.AssertionFailed.
 
-   @version $Date: 2005/02/01 23:01:35 $
+   @version $Date: 2005/04/11 13:56:13 $
    @author Daniel Bonniot (bonniot@users.sourceforge.net)
  */
 
@@ -81,7 +81,12 @@ public class Assert extends ProcedureN implements Inlineable
     else
       {
 	args[1].compile(comp, Target.pushObject);
-	code.emitInvokeSpecial(errorInitString);
+        if(args[1].getType().getName().equals(Type.string_type.getName())){
+          code.emitInvokeSpecial(errorInitString);
+        }else{
+          code.emitInvokeVirtual(errorToString);
+          code.emitInvokeSpecial(errorInitString);
+        }
       }
 
     code.emitThrow();
@@ -93,7 +98,7 @@ public class Assert extends ProcedureN implements Inlineable
   private static final ClassType 
     errorClass = ClassType.make("nice.lang.AssertionFailed");
 
-  private static Method errorInit, errorInitString;
+  private static Method errorInit, errorInitString, errorToString;
 
   private static void prepare()
   {
@@ -104,6 +109,8 @@ public class Assert extends ProcedureN implements Inlineable
       ("<init>", Access.PUBLIC, new Type[]{}, Type.void_type);
     errorInitString = errorClass.addMethod
       ("<init>", Access.PUBLIC, new Type[]{Type.string_type}, Type.void_type);
+    errorToString = Type.pointer_type.addMethod
+      ("toString", Access.PUBLIC, new Type[]{}, Type.string_type);
   }
 
   public Type getReturnType (Expression[] args)
