@@ -47,22 +47,39 @@ public class ReferenceOp extends Procedure2 implements Branchable, bossa.syntax.
   public void compile (ApplyExp exp, Compilation comp, Target target)
   {
     Expression[] args = exp.getArgs();
+    compile(args, comp, target);
+  }
+
+  void compile (Expression[] args, Compilation comp, Target target)
+  {
     CodeAttr code = comp.getCode();
-    Target stack = new StackTarget(Type.pointer_type);
+    Target stack = Target.pushObject;
     Label _else = new Label(code);
     Label _end = new Label(code);
-    
-    if (args[0] instanceof QuoteExp && 
-       ((QuoteExp)args[0]).getType() == Type.nullType)
+
+    boolean prim0 = args[0].getType() instanceof PrimType;
+    boolean prim1 = args[1].getType() instanceof PrimType;
+
+    if (prim1 && ! prim0)
+      {
+        compilePrimitive(false, args[0], args[1], comp);
+        return;
+      }
+    else if (prim0 && ! prim1)
+      {
+        compilePrimitive(true,  args[1], args[0], comp);
+        return;
+      }
+    else if (args[0] instanceof QuoteExp &&
+             ((QuoteExp)args[0]).getType() == Type.nullType)
     {
       args[1].compile(comp, stack);
       if (kind == Eq)
 	code.emitGotoIfNotNull(_else);
       else
 	code.emitGotoIfNull(_else);
-
     }
-    else if (args[1] instanceof QuoteExp && 
+    else if (args[1] instanceof QuoteExp &&
        ((QuoteExp)args[1]).getType() == Type.nullType)
     {
       args[0].compile(comp, stack);
@@ -70,7 +87,6 @@ public class ReferenceOp extends Procedure2 implements Branchable, bossa.syntax.
 	code.emitGotoIfNotNull(_else);
       else
 	code.emitGotoIfNull(_else);
-
     }
     else
     {
@@ -84,90 +100,118 @@ public class ReferenceOp extends Procedure2 implements Branchable, bossa.syntax.
     }
     code.emitPushBoolean(true);
     code.emitGoto(_end);
-    code.popType(); //simulate 'else' otherwise gnu.bytecode don't like it
+    code.popType(); // simulate 'else' otherwise gnu.bytecode doesn't like it
     _else.define(code);
     code.emitPushBoolean(false);
     _end.define(code);
-        
+
     target.compileFromStack(comp, retType);
   }
 
+  private void compilePrimitive
+    (boolean primFirst, Expression nonPrimArg, Expression primArg,
+     Compilation comp)
+  {
+    CodeAttr code = comp.getCode();
+    Target stack = Target.pushObject;
+    Target primTarget = new StackTarget(primArg.getType());
+    boolean eq = kind == Eq;
+
+    if (primFirst)
+      primArg.compile(comp, primTarget);
+    nonPrimArg.compile(comp, stack);
+    code.emitDup();
+
+    code.emitIfNotNull();
+      primTarget.compileFromStack(comp, code.topType());
+      if (! primFirst)
+        primArg.compile(comp, primTarget);
+      code.emitIfEq();
+        code.emitPushBoolean(eq);
+      code.emitElse();
+        code.emitPushBoolean(!eq);
+      code.emitFi();
+    code.emitElse();
+      code.emitPop(primFirst ? 2 : 1);
+      code.emitPushBoolean(!eq);
+    code.emitFi();
+  }
+
   public void compileJump (Compilation comp, Expression[] args, Label to)
   {
-    CodeAttr code = comp.getCode();
-    Target stack = new StackTarget(Type.pointer_type);
-
-    if (args[0] instanceof QuoteExp && 
-       ((QuoteExp)args[0]).getType() == Type.nullType)
-    {
-      args[1].compile(comp, stack);
-      if (kind == Eq)
-	code.emitGotoIfNull(to);
-      else
-	code.emitGotoIfNotNull(to);
-
-    }
-    else if (args[1] instanceof QuoteExp && 
-       ((QuoteExp)args[1]).getType() == Type.nullType)
-    {
-      args[0].compile(comp, stack);
-      if (kind == Eq)
-	code.emitGotoIfNull(to);
-      else
-	code.emitGotoIfNotNull(to);
-
-    }
-    else
-    {
-      args[0].compile(comp, stack);
-      args[1].compile(comp, stack);
-      if (kind == Eq)
-	code.emitGotoIfEq(to);
-      else
-	code.emitGotoIfNE(to);
-
-    }
+    compileJump(comp, args, to, false);
   }
 
   public void compileJumpNot (Compilation comp, Expression[] args, Label to)
   {
-    CodeAttr code = comp.getCode();
-    Target stack = new StackTarget(Type.pointer_type);
+    compileJump(comp, args, to, true);
+  }
 
-    if (args[0] instanceof QuoteExp && 
+  private void compileJump (Compilation comp, Expression[] args, Label to,
+                            boolean not)
+  {
+    CodeAttr code = comp.getCode();
+    Target stack = Target.pushObject;
+
+    boolean prim0 = args[0].getType() instanceof PrimType;
+    boolean prim1 = args[1].getType() instanceof PrimType;
+
+    if (prim1 && ! prim0 || prim0 && ! prim1)
+      {
+        compile(args, comp, StackTarget.getInstance(Type.boolean_type));
+        if (not)
+          code.emitGotoIfIntEqZero(to);
+        else
+          code.emitGotoIfIntNeZero(to);
+      }
+
+    else if (args[0] instanceof QuoteExp &&
        ((QuoteExp)args[0]).getType() == Type.nullType)
     {
-      args[1].compile(comp, stack);
-      if (kind == Eq)
-	code.emitGotoIfNotNull(to);
-      else
-	code.emitGotoIfNull(to);
+      if (args[1].getType() instanceof PrimType)
+        {
+          if (not ^ kind == Ne)
+            code.emitGoto(to);
 
+          return;
+        }
+
+      args[1].compile(comp, stack);
+      if (not ^ kind == Eq)
+	code.emitGotoIfNull(to);
+      else
+	code.emitGotoIfNotNull(to);
     }
-    else if (args[1] instanceof QuoteExp && 
+    else if (args[1] instanceof QuoteExp &&
        ((QuoteExp)args[1]).getType() == Type.nullType)
     {
-      args[0].compile(comp, stack);
-      if (kind == Eq)
-	code.emitGotoIfNotNull(to);
-      else
-	code.emitGotoIfNull(to);
+      if (args[0].getType() instanceof PrimType)
+        {
+          if (not ^ kind == Ne)
+            code.emitGoto(to);
 
+          return;
+        }
+
+      args[0].compile(comp, stack);
+      if (not ^ kind == Eq)
+	code.emitGotoIfNull(to);
+      else
+	code.emitGotoIfNotNull(to);
     }
     else
     {
       args[0].compile(comp, stack);
       args[1].compile(comp, stack);
-      if (kind == Eq)
-	code.emitGotoIfNE(to);
-      else
+      if (not ^ kind == Eq)
 	code.emitGotoIfEq(to);
-
+      else
+	code.emitGotoIfNE(to);
     }
   }
 
   private static final Type retType = Type.boolean_type;
-  
+
   public Type getReturnType (Expression[] args)
   {
     return retType;
@@ -176,12 +220,11 @@ public class ReferenceOp extends Procedure2 implements Branchable, bossa.syntax.
   public void checkSpecialRequirements(bossa.syntax.Expression[] arguments)
   {
     bossa.syntax.Expression exp = null;
-    if (arguments[0] instanceof bossa.syntax.NullExp)
+    if (arguments[0].isNull())
       exp = arguments[1];
-    else if (arguments[1] instanceof bossa.syntax.NullExp)
+    else if (arguments[1].isNull())
       exp = arguments[0];
 
- 
     if (exp != null && nice.tools.typing.Types.isSure(exp.getType().getMonotype()))
       bossa.util.User.warning(exp, "Comparing a non-null value with null");
   }
